@@ -7,11 +7,12 @@ from progress_table import ProgressTable
 # Define the columns at the beginning
 table = ProgressTable(
     columns=["step", "x", "x squared"],
-    # Default values:
+
+    # Default arguments:
     refresh_rate=10,
     num_decimal_places=4,
     default_column_width=8,
-    default_column_alignment="centera",
+    default_column_alignment="center",
     print_row_on_update=True,
     reprint_header_every_n_rows=30,
     custom_format=None,
@@ -23,6 +24,11 @@ table.add_column("x", width=3)
 table.add_column("x root", color="red")
 table.add_column("random average", color=["bright", "red"], aggregate="mean")
 
+for _ in range(20):
+    print()
+
+time.sleep(1)
+
 for step in range(10):
     x = random.randint(0, 200)
 
