@@ -3,6 +3,7 @@
 from __future__ import annotations
 
 import functools
+import inspect
 import logging
 import math
 import sys
@@ -15,9 +16,13 @@ from colorama import Fore, Style
 
 from . import symbols
 
-ALL_COLORS = [getattr(Fore, x) for x in dir(Fore) if not x.startswith("__")]
-ALL_STYLES = [getattr(Style, x) for x in dir(Style) if not x.startswith("__")]
-ALL_COLOR_STYLES = ALL_COLORS + ALL_STYLES
+ALL_COLOR_NAMES = [x for x in dir(Fore) if not x.startswith("__")]
+ALL_STYLE_NAMES = [x for x in dir(Style) if not x.startswith("__")]
+ALL_COLORS_STYLES_NAMES = ALL_COLOR_NAMES + ALL_STYLE_NAMES
+
+ALL_COLORS = [getattr(Fore, x) for x in ALL_COLOR_NAMES]
+ALL_STYLES = [getattr(Style, x) for x in ALL_STYLE_NAMES]
+ALL_COLORS_STYLES = ALL_COLORS + ALL_STYLES
 
 ITERATOR_LENGTH_UNKNOWN_WARNED_ONCE = False
 ITERATOR_LENGTH_CACHE: Dict[int, int] = {}
@@ -38,6 +43,42 @@ class ProgressTable:
         table_style="normal",
         file=sys.stdout,
     ):
+        """Progress Table instance.
+
+        Columns have to be specified before displaying the first row or launching a progress bar!
+
+        Args:
+            columns: Columns that will be displayed in the header. Columns can be provided directly in
+                     `__init__` or through methods `add_column` and `add_columns`. Columns added through
+                     `__init__` will have default settings like alignment, color and width, while
+                     columns added through methods can have those customized.
+            refresh_rate: The maximal number of times per second the last row of the Table will be refreshed.
+                          This applies only when using Progress Bar or when `print_row_on_update = True`.
+            num_decimal_places: This is only applicable when using the default formatting. This won't be used
+                                if `custom_format` is set. If applicable, for every displayed value except
+                                integers there will be an attempt to round it.
+            default_column_width: Width of the column excluding cell padding. This can be overwritten by
+                                  columns by using an argument in `add_column` method.
+            default_column_alignment: Alignment in the column. Can be aligned either to `left`, `right` or
+                                      `center` (default). This can be overwritten by columns by using
+                                      an argument in `add_column` method.
+            print_row_on_update: True by default. If False, the current row will be displayed only when
+                                 it's ready. Row is considered ready after calling `.next_row` or `.close`
+                                 methods.
+            reprint_header_every_n_rows: 30 by default. When table has a lot of rows, it can be useful to
+                                         remind what the header is. If True, hedaer will be displayed
+                                         periodically after the selected number of rows. 0 to supress.
+            custom_format: A function that allows specyfing custom formatting for values in cells.
+                           This function should be universal and work for all datatypes as inputs.
+                           It takes one value as an input and returns one value as an output.
+            embedded_progress_bar: False by default. If True, changes the way the progress bar looks.
+                                   Embedded version is more subtle, but does not prevent the current row
+                                   from being displayed. If False, the progress bar covers the current
+                                   row, preventing the user from seeing values that are being updated
+                                   until the progress bar finishes.
+            table_style: Change the borders of the table. Cause KeyError to see all the available styles.
+            file: Redirect the output to another stream. Defaults to sys.stdout.
+        """
         self.refresh_rate = refresh_rate
         self.default_width = default_column_width
         self.default_alignment = default_column_alignment
@@ -69,21 +110,15 @@ class ProgressTable:
         self.progress_bar_active = False
         self._refresh_progress_bar: Callable = lambda: None
 
-        self._symbols: symbols.Symbols
+        allowed_styles = [getattr(symbols, class_name) for class_name in dir(symbols)]
+        allowed_styles = [x for x in allowed_styles if inspect.isclass(x) and issubclass(x, symbols.Symbols)]
+        style_name_to_style = {x.name: x for x in allowed_styles if hasattr(x, "name")}
 
-        if table_style == "normal":
-            self._symbols = symbols.SymbolsUnicode()
-        elif table_style == "round":
-            self._symbols = symbols.SymbolsUnicodeRound()
-        elif table_style == "double":
-            self._symbols = symbols.SymbolsUnicodeDouble()
-        elif table_style == "bold":
-            self._symbols = symbols.SymbolsUnicodeBold()
-        elif table_style == "basic":
-            self._symbols = symbols.SymbolsBasic()
+        if table_style in style_name_to_style:
+            self._symbols = style_name_to_style[table_style]
         else:
-            allowed_styles = ["normal", "round", "double", "bold", "basic"]
-            raise KeyError(f"Style '{table_style}' not in {allowed_styles}!")
+            allowed_style_names = list(style_name_to_style)
+            raise KeyError(f"Style '{table_style}' not in {allowed_style_names}!")
 
         for column in columns:
             self.add_column(column)
@@ -101,7 +136,130 @@ class ProgressTable:
 
         return fmt
 
-    def apply_cell_formatting(self, str_value: str, column: str):
+    def add_column(self, name, *, width=None, alignment=None, color=None, aggregate=None):
+        """Add column to the table.
+
+        Args:
+            name: Name of the column. Will be displayed in the header.
+            width: Width of the column. If width is smaller than the name, width will be automatically
+                   increased to the smallest possible value. This means setting `width=0` will set the column
+                   to the smallest possible width allowed by `name`.
+            alignment: Alignment of the data in the column.
+            color: Color of the header and the data in the column.
+            aggregate: By default, there's no aggregation. But if this is for example 'mean', then
+                       after every update in the current row, the mean of the provided values will be
+                       displayed. Aggregation is reset at every new row.
+        """
+        assert not self.header_printed, "Columns cannot be modified after displaying the first row!"
+
+        if name in self.columns:
+            logging.info(f"Column '{name}' already exists!")
+        else:
+            self.columns.append(name)
+
+        self._colors.setdefault(name, "")
+        if color is not None:
+            if isinstance(color, str):
+                color = [color]
+            for str_color in color:
+                byte_color = self._maybe_convert_to_colorama(str_color)
+                self._check_color(byte_color, str_color)
+                self._colors[name] += byte_color
+
+        alignment = alignment if alignment is not None else self.default_alignment
+        self._alignment[name] = alignment
+
+        assert aggregate in [None, "mean", "sum"], "Allowed aggregate types: [None, 'mean', 'sum']"
+        self._aggregate[name] = aggregate
+
+        width = width if width is not None else self.default_width
+        if width < len(name):
+            width = len(name)
+        self._widths[name] = width
+
+    def add_columns(self, iterable, **kwds):
+        """Add multiple columns to the table."""
+        for column in iterable:
+            self.add_column(column, **kwds)
+
+    def next_row(self, save=True):
+        """End the current row."""
+        self._print_row()
+        self._maybe_line_ending()
+        if self.progress_bar_active:
+            self._refresh_progress_bar()
+        sys.stdout.flush()
+
+        if save and len(self._new_row) > 0:
+            self.finished_rows.append(self._new_row)
+            self.num_rows += 1
+
+        self._new_row = defaultdict(str)
+        self._aggregate_n = defaultdict(int)
+
+    def close(self):
+        """End the table and close it."""
+        self.next_row()
+        self._print_bottom_bar()
+        self.header_printed = False
+        self._print()
+
+    def to_list(self):
+        """Convert to Python nested lists."""
+        return [[row[col] for col in self.columns] for row in self.finished_rows]
+
+    def to_numpy(self):
+        """Convert to numpy array."""
+        import numpy as np
+
+        return np.array(self.to_list())
+
+    def to_df(self):
+        """Convert to pandas DataFrame."""
+        import pandas as pd
+
+        return pd.DataFrame(self.to_list(), columns=self.columns)
+
+    def display(self):
+        """Display the whole table. Can be used after closing the table."""
+        if self.header_printed:
+            self.close()
+        self._display_custom(self.to_list())
+
+    def update(self, key, value, *, weight=1):
+        """Update value in the current row."""
+        assert key in self.columns, f"Column '{key}' not in {self.columns}"
+
+        if self._aggregate[key] == "sum":
+            aggr_value = self._new_row[key] if self._aggregate_n[key] > 0 else 0
+            self._new_row[key] = aggr_value + value * weight
+            self._aggregate_n[key] += weight
+
+        elif self._aggregate[key] == "mean":
+            n = self._aggregate_n[key]
+            aggr_value = self._new_row[key] if n > 0 else 0
+            self._new_row[key] = (aggr_value * n + value * weight) / (n + weight)
+            self._aggregate_n[key] += weight
+
+        else:
+            self._new_row[key] = value
+
+        t0 = time.time()
+        td = t0 - self._last_time_row_printed
+        if self.print_row_on_update and td > 1 / self.refresh_rate:
+            self._last_time_row_printed = t0
+
+            if not self.progress_bar_active:
+                self._print_row()
+            elif self.embedded_progress_bar:
+                self._refresh_progress_bar()
+
+    def update_from_dict(self, dictionary):
+        """Update multiple values in the current row."""
+        for key, value in dictionary.items():
+            self.update(key, value)
+
+    def _apply_cell_formatting(self, str_value: str, column: str):
         width = self._widths[column]
 
         alignment = self._alignment[column]
@@ -128,76 +286,6 @@ class ProgressTable:
             str_value = f"{self._colors[column]}{str_value}{Style.RESET_ALL}"
         return str_value
 
-    def add_column(self, name, *, width=None, alignment=None, color=None, aggregate=None):
-        assert not self.header_printed, "Columns cannot be modified after displaying the first row!"
-
-        if name in self.columns:
-            logging.info(f"Column '{name}' already exists!")
-        else:
-            self.columns.append(name)
-
-        width = width if width is not None else self.default_width
-        if width < len(name):
-            width = len(name)
-
-        self._colors.setdefault(name, "")
-        if color is not None:
-            if isinstance(color, str):
-                color = [color]
-            for str_color in color:
-                byte_color = self._maybe_convert_to_colorama(str_color)
-                self._check_color(byte_color, str_color)
-                self._colors[name] += byte_color
-
-        alignment = alignment if alignment is not None else self.default_alignment
-        self._alignment[name] = alignment
-
-        assert aggregate in [None, "mean", "sum"], "Allowed aggregate types: [None, 'mean', 'sum']"
-        self._aggregate[name] = aggregate
-        self._widths[name] = width
-
-    def add_columns(self, iterable, **kwds):
-        for column in iterable:
-            self.add_column(column, **kwds)
-
-    def next_row(self, save=True):
-        self._print_row()
-        self._maybe_line_ending()
-        if self.progress_bar_active:
-            self._refresh_progress_bar()
-        sys.stdout.flush()
-
-        if save and len(self._new_row) > 0:
-            self.finished_rows.append(self._new_row)
-            self.num_rows += 1
-
-        self._new_row = defaultdict(str)
-        self._aggregate_n = defaultdict(int)
-
-    def close(self):
-        self.next_row()
-        self._print_bottom_bar()
-        self.header_printed = False
-        self._print()
-
-    def to_list(self):
-        return [[row[col] for col in self.columns] for row in self.finished_rows]
-
-    def to_numpy(self):
-        import numpy as np
-
-        return np.array(self.to_list())
-
-    def to_df(self):
-        import pandas as pd
-
-        return pd.DataFrame(self.to_list(), columns=self.columns)
-
-    def display(self):
-        if self.header_printed:
-            self.close()
-        self._display_custom(self.to_list())
-
     def _bar(self, left: str, center: str, right: str):
         content_list: List[str] = []
         for col in self.columns:
@@ -264,7 +352,9 @@ class ProgressTable:
     def _check_color(color, color_str=None):
         if color_str is None:
             color_str = color
-        assert color in ALL_COLOR_STYLES, f"Only colorama colors are allowed (not {color_str})!"
+        assert (
+            color in ALL_COLORS_STYLES
+        ), f"Only colorama colors are allowed, not '{color_str}'! Available: {ALL_COLORS_STYLES_NAMES}"
 
     def _print_header(self, top=True):
         assert self.columns, "Columns are required! Use .add_column method or specify them in __init__!"
@@ -277,7 +367,7 @@ class ProgressTable:
 
         content = []
         for column in self.columns:
-            value = self.apply_cell_formatting(column, column)
+            value = self._apply_cell_formatting(column, column)
             content.append(value)
         s = "".join(
             ["\r", self._symbols.vertical, self._symbols.vertical.join(content), self._symbols.vertical]
@@ -294,7 +384,7 @@ class ProgressTable:
         for column in self.columns:
             value = self._new_row[column]
             value = self.custom_format(value)
-            value = self.apply_cell_formatting(str_value=str(value), column=column)
+            value = self._apply_cell_formatting(str_value=str(value), column=column)
             content.append(value)
         return "".join(
             ["\r", self._symbols.vertical, self._symbols.vertical.join(content), self._symbols.vertical]
@@ -303,8 +393,10 @@ class ProgressTable:
     def _print_row(self):
         if not self.header_printed:
             self._print_header(top=True)
-        if self.num_rows - self._last_header_printed_at_row_count >= self.reprint_header_every_n_rows:
-            self._print_header(top=False)
+
+        if self.reprint_header_every_n_rows != 0:
+            if self.num_rows - self._last_header_printed_at_row_count >= self.reprint_header_every_n_rows:
+                self._print_header(top=False)
 
         if len(self._new_row) == 0:
             return
@@ -361,7 +453,7 @@ class ProgressTable:
         self.close()
 
     def __call__(self, iterator, length=None, prefix="", show_throughput=True):
-        """Display progress bar over the iterator."""
+        """Display progress bar over the iterator. Try to figure out the iterator length."""
         global ITERATOR_LENGTH_UNKNOWN_WARNED_ONCE, ITERATOR_LENGTH_CACHE
 
         if not self.header_printed:
@@ -424,38 +516,6 @@ class ProgressTable:
         self.progress_bar_active = False
         self._print_row()
 
-    def update_from_dict(self, dictionary):
-        for key, value in dictionary.items():
-            self.update(key, value)
-
-    def update(self, key, value, *, weight=1):
-        """Update value in the current row."""
-        assert key in self.columns, f"Column '{key}' not in {self.columns}"
-
-        if self._aggregate[key] == "sum":
-            aggr_value = self._new_row[key] if self._aggregate_n[key] > 0 else 0
-            self._new_row[key] = aggr_value + value * weight
-            self._aggregate_n[key] += weight
-
-        elif self._aggregate[key] == "mean":
-            n = self._aggregate_n[key]
-            aggr_value = self._new_row[key] if n > 0 else 0
-            self._new_row[key] = (aggr_value * n + value * weight) / (n + weight)
-            self._aggregate_n[key] += weight
-
-        else:
-            self._new_row[key] = value
-
-        t0 = time.time()
-        td = t0 - self._last_time_row_printed
-        if self.print_row_on_update and td > 1 / self.refresh_rate:
-            self._last_time_row_printed = t0
-
-            if not self.progress_bar_active:
-                self._print_row()
-            elif self.embedded_progress_bar:
-                self._refresh_progress_bar()
-
     def __setitem__(self, key, value):
         self.update(key, value, weight=1)
 
